﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Empties the given stack.
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class FlushExpression<T> : StatelessExpression
    {
        public void Eval(IStack<T> stack)
        {
            stack.Clear();
        }
    }

    public class IntegerFlushExpression : FlushExpression<long>
    {
        protected override string InitStringRepresentation() { return "INTEGER.FLUSH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack);
        }
    }

    public class FloatFlushExpression : FlushExpression<double>
    {
        protected override string InitStringRepresentation() { return "FLOAT.FLUSH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.FloatStack);
        }
    }

    public class BooleanFlushExpression : FlushExpression<bool>
    {
        protected override string InitStringRepresentation() { return "BOOLEAN.FLUSH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack);
        }
    }

    public class NameFlushExpression : FlushExpression<string>
    {
        protected override string InitStringRepresentation() { return "NAME.FLUSH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.NameStack);
        }
    }

    public class ExecFlushExpression : FlushExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "EXEC.FLUSH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.ExecStack);
        }
    }

    public class CodeFlushExpression : FlushExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "CODE.FLUSH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.CodeStack);
        }
    }
}
