﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class RotateExpression<T> : StatelessExpression
    {
        public void Eval(IStack<T> stack)
        {
            if (stack.Count < 3)
                return;

            stack.Swap(3);
        }
    }

    public class IntegerRotateExpression : RotateExpression<long>
    {
        protected override string InitStringRepresentation() { return "INTEGER.ROT"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack);
        }
    }

    public class FloatRotateExpression : RotateExpression<double>
    {
        protected override string InitStringRepresentation() { return "FLOAT.ROT"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.FloatStack);
        }
    }

    public class BooleanRotateExpression : RotateExpression<bool>
    {
        protected override string InitStringRepresentation() { return "BOOLEAN.ROT"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack);
        }
    }

    public class NameRotateExpression : RotateExpression<string>
    {
        protected override string InitStringRepresentation() { return "NAME.ROT"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.NameStack);
        }
    }

    public class ExecRotateExpression : RotateExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "EXEC.ROT"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.ExecStack);
        }
    }

    public class CodeRotateExpression : RotateExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "CODE.ROT"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.CodeStack);
        }
    }
}
